"use strict";

/* exported logger */

var Panel = require('../Panel');

var DualListbox = require('dual-listbox').DualListbox;
var logger = require('../../logger');
var Functions = require('../../Functions');

var Promise = require("bluebird");

function ElementExportPanel(params) {
  params.panelName = "elementExport";
  params.scrollable = true;
  Panel.call(this, params);

  var self = this;
  self._createGui();
}

ElementExportPanel.prototype = Object.create(Panel.prototype);
ElementExportPanel.prototype.constructor = ElementExportPanel;

function compareSimple(val1, val2) {
  if (val1 < val2) {
    return -1;
  } else if (val1 > val2) {
    return 1;
  } else {
    return 0;
  }
}

ElementExportPanel.prototype._createGui = function() {

};

ElementExportPanel.prototype.init = function() {
  var self = this;
  var element = self.getElement();
  var configuration;
  return ServerConnector.getConfiguration().then(function(result) {
    configuration = result;
    var typeDiv = self._createSelectTypeDiv(configuration.getElementTypes());
    element.appendChild(typeDiv);
    element.appendChild(self._createSelectColumnDiv());
    return ServerConnector.getProjectStatistics(self.getProject().getProjectId());
  }).then(function(statistics) {
    var miriamDiv = self._createMiriamTypeDiv(statistics);
    element.appendChild(miriamDiv);
    self.setMiriamTypesDualListbox(new DualListbox("#miriamTypeSelect", {
      availableTitle : 'Available',
      selectedTitle : 'Used',
      addButtonText : '>',
      removeButtonText : '<',
      addAllButtonText : '>>',
      removeAllButtonText : '<<'
    }));
  }).then(function() {
    return self._createSelectIncludedCompartmentDiv();
  }).then(function(selectIncludedCompartmentDiv) {
    element.appendChild(selectIncludedCompartmentDiv);
    self.setIncludedCompartmentsDualListbox(new DualListbox("#includedCompartmentsSelect", {
      availableTitle : 'Available',
      selectedTitle : 'Used',
      addButtonText : '>',
      removeButtonText : '<',
      addAllButtonText : '>>',
      removeAllButtonText : '<<'
    }));
    return self._createSelectExcludedCompartmentDiv();
  }).then(function(selectExcludedCompartmentDiv) {
    element.appendChild(selectExcludedCompartmentDiv);
    self.setExcludedCompartmentsDualListbox(new DualListbox("#excludedCompartmentsSelect", {
      availableTitle : 'Available',
      selectedTitle : 'Used',
      addButtonText : '>',
      removeButtonText : '<',
      addAllButtonText : '>>',
      removeAllButtonText : '<<'
    }));
    element.appendChild(self._createDownloadButton());
  }).then(function() {
    $(window).trigger('resize');
  });
};

ElementExportPanel.prototype._createSelectTypeDiv = function(elementTypes) {
  var typeDiv = Functions.createElement({
    type : "div",
    name : "typeSelectDiv",
  });
  typeDiv.appendChild(Functions.createElement({
    type : "h4",
    content : " TYPE:",
  }));
  var choicesContainer = Functions.createElement({
    type : "ul",
    className : "minerva-checkbox-grid"
  });
  typeDiv.appendChild(choicesContainer);
  for (var i = 0; i < elementTypes.length; i++) {
    var elementType = elementTypes[i];
    var row = Functions.createElement({
      type : "li",
      content : "<div class=\"checkbox\"><label> <input type=\"checkbox\" name=\"" + elementType + "\" value=\""
          + elementType + "\" />" + elementType + "</label></div>"
    });
    choicesContainer.appendChild(row);
  }
  choicesContainer.appendChild(Functions.createElement({
    type : "li",
    content : "<div class=\"checkbox\"><label> <input type=\"checkbox\" name=\"ALL\" value=\"ALL\" />ALL</label></div>"
  }));
  return typeDiv;
};

ElementExportPanel.prototype._createMiriamTypeDiv = function(statistics) {
  var typeDiv = Functions.createElement({
    type : "div",
    name : "miriamSelectDiv",
    className : "minerva-export-dual-listbox-container",
  });
  typeDiv.appendChild(Functions.createElement({
    type : "h4",
    content : " Annotations:",
  }));

  var annotations = statistics.getElementAnnotations();

  var selectElement = Functions.createElement({
    type : "select",
    className : "minerva-multi-select",
    id : "miriamTypeSelect",
  });
  typeDiv.appendChild(selectElement);

  function compare(entry1, entry2) {
    if ((entry1.count === 0 && entry2.count === 0) || (entry1.count !== 0 && entry2.count !== 0)) {
      return compareSimple(entry1.miriamType.getCommonName(), entry2.miriamType.getCommonName());
    } else {
      return compareSimple(-entry1.count, -entry2.count);
    }
  }
  annotations.sort(compare);

  for (var i = 0; i < annotations.length; i++) {
    var miriamType = annotations[i].miriamType;
    var count = annotations[i].count;

    var option = new Option();
    option.value = miriamType.getName();
    if (count > 0) {
      option.innerHTML = "<div class='minerva-multi-select-special'>" + miriamType.getCommonName() + " (" + count
          + ")</div>";
    } else {
      option.innerHTML = "<div >" + miriamType.getCommonName() + "</div>";
    }
    selectElement.appendChild(option);
  }
  return typeDiv;
};

ElementExportPanel.prototype._createDownloadButton = function() {
  var self = this;
  var downloadDiv = Functions.createElement({
    type : "div",
    name : "downloadDiv",
    style : "clear:both; padding: 10px;",
  });
  var button = Functions.createElement({
    type : "button",
    content : " Download",
    onclick : function() {
      return self.createResponseString().then(function(result) {
        logger.info(result);
      });
    },
  });
  downloadDiv.appendChild(button);

  return downloadDiv;
};

ElementExportPanel.prototype.createResponseString = function() {
  var self = this;
  var types;
  var models = [ self.getProject().getModel() ];
  for (var i = 0; i < self.getProject().getModel().getSubmodels().length; i++) {
    models.push(self.getProject().getModel().getSubmodels()[i]);
  }

  var elements = [];
  return self.getSelectedTypes().then(function(result) {
    types = result;
    var promises = [];
    for (var i = 0; i < models.length; i++) {
      promises.push(models[i].getAliases({
        type : types,
        complete : true,
      }));
    }
    return Promise.all(promises);
  }).then(function(aliasesByModel) {
    for (var i = 0; i < aliasesByModel.length; i++) {
      for (var j = 0; j < aliasesByModel[i].length; j++) {
        elements.push(aliasesByModel[i][j]);
      }
    }
    return self.getSelectedColumns();
  }).then(function(selectedColumns) {

    var rows = [];
    rows.push(self.createResponseHeader(selectedColumns));
    for (var i = 0; i < elements.length; i++) {
      rows.push(self.createResponseRow(elements[i], selectedColumns));
    }
    return rows.join("\n");
  });
};

ElementExportPanel.prototype.createResponseHeader = function(columns) {
  var stringBuilder = [];
  for (var i = 0; i < columns.length; i++) {
    var column = columns[i];
    stringBuilder.push(column.name);
  }
  var result = stringBuilder.join("\t");
  return result;

};

ElementExportPanel.prototype.createResponseRow = function(alias, columns) {
  var stringBuilder = [];
  for (var i = 0; i < columns.length; i++) {
    var column = columns[i];
    var value = alias[column.method]();
    if (column.formatFunction !== undefined) {
      value = column.formatFunction(value);
    }
    stringBuilder.push(value);

  }
  var result = stringBuilder.join("\t");
  return result;
};

ElementExportPanel.prototype._createSelectIncludedCompartmentDiv = function() {
  var self = this;
  var typeDiv = Functions.createElement({
    type : "div",
    name : "includedCompartmentSelectDiv",
    className : "minerva-export-dual-listbox-container",
  });
  typeDiv.appendChild(Functions.createElement({
    type : "h4",
    content : " Included compartment/pathways:",
  }));

  return self._getCompartmentNames().then(function(compartmentNames) {

    var selectElement = Functions.createElement({
      type : "select",
      className : "minerva-multi-select",
      id : "includedCompartmentsSelect",
    });
    typeDiv.appendChild(selectElement);

    for (var i = 0; i < compartmentNames.length; i++) {
      var name = compartmentNames[i];

      var option = new Option();
      option.value = name;
      option.innerHTML = "<div >" + name + "</div>";
      selectElement.appendChild(option);
    }
    return typeDiv;
  });
};

ElementExportPanel.prototype._createSelectExcludedCompartmentDiv = function() {
  var self = this;
  var typeDiv = Functions.createElement({
    type : "div",
    name : "excludedCompartmentSelectDiv",
    className : "minerva-export-dual-listbox-container",
  });
  typeDiv.appendChild(Functions.createElement({
    type : "h4",
    content : " Excluded compartment/pathways:",
  }));

  return self._getCompartmentNames().then(function(compartmentNames) {

    var selectElement = Functions.createElement({
      type : "select",
      className : "minerva-multi-select",
      id : "excludedCompartmentsSelect",
    });
    typeDiv.appendChild(selectElement);

    for (var i = 0; i < compartmentNames.length; i++) {
      var name = compartmentNames[i];

      var option = new Option();
      option.value = name;
      option.innerHTML = "<div >" + name + "</div>";
      selectElement.appendChild(option);
    }
    return typeDiv;
  });
};

ElementExportPanel.prototype._getAllCompartments = function() {
  var self = this;
  var compartments = [];
  var i;

  var models = [ self.getProject().getModel() ];
  for (i = 0; i < self.getProject().getModel().getSubmodels().length; i++) {
    models.push(self.getProject().getModel().getSubmodels()[i]);
  }

  var promises = [];
  for (i = 0; i < models.length; i++) {
    promises.push(models[i].getCompartments());
  }

  return Promise.all(promises).then(function(result) {
    for (var i = 0; i < result.length; i++) {
      var modelCompartments = result[i];
      for (var j = 0; j < modelCompartments.length; j++) {
        compartments.push(modelCompartments[j]);
      }
    }
    return compartments;
  });
};
ElementExportPanel.prototype._getCompartmentNames = function() {
  var self = this;
  var compartments = [];
  return self._getAllCompartments().then(function(result) {

    var addedNames = [];
    for (var i = 0; i < result.length; i++) {
      if (addedNames[result[i].getName()] === undefined) {
        compartments.push(result[i].getName());
        addedNames[result[i].getName()] = true;
      }
    }
    compartments.sort(compareSimple);
    return compartments;
  });
};

ElementExportPanel.prototype._createSelectColumnDiv = function() {
  var self = this;
  var columnDiv = Functions.createElement({
    type : "div",
    name : "columnSelectDiv",
  });
  columnDiv.appendChild(Functions.createElement({
    type : "h4",
    content : " COLUMN:",
  }));
  var choicesContainer = Functions.createElement({
    type : "ul",
    className : "minerva-checkbox-grid"
  });
  var columnTypes = self.getAllColumns();

  columnDiv.appendChild(choicesContainer);
  for (var i = 0; i < columnTypes.length; i++) {
    var columnType = columnTypes[i];
    var row = Functions.createElement({
      type : "li",
      content : "<div class=\"checkbox\"><label> <input type=\"checkbox\" name=\"column_" + columnType.columnName
          + "\" value=\"" + columnType.columnName + "\" />" + columnType.name + "</label></div>"
    });
    choicesContainer.appendChild(row);
  }
  choicesContainer.appendChild(Functions.createElement({
    type : "li",
    content : "<div class=\"checkbox\"><label> <input type=\"checkbox\" name=\"ALL\" value=\"ALL\" />ALL</label></div>"
  }));
  return columnDiv;
};

ElementExportPanel.prototype.getAllColumns = function() {
  return [ {
    "columnName" : "id",
    "method" : "getId",
    "name" : "Id",
  }, {
    "columnName" : "name",
    "method" : "getName",
    "name" : "Name",
  }, {
    "columnName" : "description",
    "method" : "getDescription",
    "name" : "Description",
  }, {
    "columnName" : "modelId",
    "method" : "getModelId",
    "name" : "Model",
  }, {
    "columnName" : "type",
    "method" : "getType",
    "name" : "Type",
  }, {
    "columnName" : "complexId",
    "method" : "getComplexId",
    "name" : "Complex",
  }, {
    "columnName" : "compartmentId",
    "method" : "getCompartmentId",
    "name" : "Compartment",
  }, {
    "columnName" : "charge",
    "method" : "getCharge",
    "name" : "Charge",
  }, {
    "columnName" : "symbol",
    "method" : "getSymbol",
    "name" : "Symbol",
  }, {
    "columnName" : "fullName",
    "method" : "getFullName",
    "name" : "Full name",
  }, {
    "columnName" : "abbreviation",
    "method" : "getAbbreviation",
    "name" : "Abbreviation",
  }, {
    "columnName" : "formula",
    "method" : "getFormula",
    "name" : "Formula",
  }, {
    "columnName" : "synonyms",
    "method" : "getSynonyms",
    "name" : "Synonyms",
  }, {
    "columnName" : "formerSymbols",
    "method" : "getFormerSymbols",
    "name" : "Former symbols",
  }, {
    "columnName" : "references",
    "method" : "getReferences",
    "name" : "References",
    "formatFunction" : function(references) {
      var stringBuilder = [];
      for (var i = 0; i < references.length; i++) {
        var reference = references[i];
        stringBuilder.push(reference.getType() + ":" + reference.getResource());
      }
      return stringBuilder.join(",");
    },
  }, {
    "columnName" : "linkedSubmodelId",
    "method" : "getLinkedSubmodelId",
    "name" : "Linked submodel",
  }, {
    "columnName" : "elementId",
    "method" : "getElementId",
    "name" : "Element external id",
  }, ];
};

ElementExportPanel.prototype.getSelectedTypes = function() {
  var self = this;

  var div = $("div[name='typeSelectDiv']", $(self.getElement()))[0];
  var result = [];
  var selectedAll = false;
  $(":checked", $(div)).each(function(index, element) {
    if (element.value === "ALL") {
      selectedAll = true;
    }
    result.push(element.value);
  });
  if (selectedAll) {
    return ServerConnector.getConfiguration().then(function(configuration) {
      return configuration.getElementTypes();
    });
  }

  return Promise.resolve(result);
};

ElementExportPanel.prototype.setMiriamTypesDualListbox = function(dualListbox) {
  this._miriamTypesDualListbox = dualListbox;
};
ElementExportPanel.prototype.getMiriamTypesDualListbox = function() {
  return this._miriamTypesDualListbox;
};

ElementExportPanel.prototype.setIncludedCompartmentsDualListbox = function(dualListbox) {
  this._includedCompartmentsDualListbox = dualListbox;
};
ElementExportPanel.prototype.getIncludedCompartmentsDualListbox = function() {
  return this._includedCompartmentsDualListbox;
};

ElementExportPanel.prototype.getSelectedIncludedCompartments = function() {
  var self = this;
  var list = self.getIncludedCompartmentsDualListbox().selected;
  var names = [];
  var result = [];
  for (var i = 0; i < list.length; i++) {
    var element = list[i];
    names[element.dataset.id] = true;
  }
  return self._getAllCompartments().then(function(compartments) {
    for (var i = 0; i < compartments.length; i++) {
      var compartment = compartments[i];
      if (names[compartment.getName()]) {
        result.push(compartment);
      }
    }
    return result;
  });
};

ElementExportPanel.prototype.setExcludedCompartmentsDualListbox = function(dualListbox) {
  this._excludedCompartmentsDualListbox = dualListbox;
};
ElementExportPanel.prototype.getExcludedCompartmentsDualListbox = function() {
  return this._excludedCompartmentsDualListbox;
};

ElementExportPanel.prototype.getSelectedExcludedCompartments = function() {
  var self = this;
  var list = self.getExcludedCompartmentsDualListbox().selected;
  var names = [];
  var result = [];
  for (var i = 0; i < list.length; i++) {
    var element = list[i];
    names[element.dataset.id] = true;
  }
  return self._getAllCompartments().then(function(compartments) {
    for (var i = 0; i < compartments.length; i++) {
      var compartment = compartments[i];
      if (names[compartment.getName()]) {
        result.push(compartment);
      }
    }
    return result;
  });
};

ElementExportPanel.prototype.getSelectedMiriamTypes = function() {
  var self = this;
  return ServerConnector.getConfiguration().then(function(configuration) {
    var selected = self.getMiriamTypesDualListbox().selected;
    var result = [];
    for (var i = 0; i < selected.length; i++) {
      var miriamType = configuration.getMiriamTypeByName(selected[i].dataset.id);
      result.push(miriamType);
    }
    return result;
  });
};

ElementExportPanel.prototype.getSelectedColumns = function() {
  var self = this;

  var div = $("div[name='columnSelectDiv']", $(self.getElement()))[0];
  var selectedColumnMap = [];
  var selectedAll = false;
  $(":checked", $(div)).each(function(index, element) {
    if (element.value === "ALL") {
      selectedAll = true;
    }
    selectedColumnMap[element.value] = true;
  });
  var columnTypes = self.getAllColumns();

  if (selectedAll) {
    return Promise.resolve(columnTypes);
  }

  var result = [];

  for (var i = 0; i < columnTypes.length; i++) {
    var columnType = columnTypes[i];
    if (selectedColumnMap[columnType.columnName] === true) {
      result.push(columnType);
    }
  }
  return Promise.resolve(result);
};

module.exports = ElementExportPanel;
