"use strict";

var Promise = require("bluebird");

var AbstractInfoWindow = require('./AbstractInfoWindow');
var Reaction = require('../data/Reaction');

/**
 * Class representing info window that should be opened when clicking on
 * reaction.
 */
function ReactionInfoWindow(params) {
  // call super constructor
  AbstractInfoWindow.call(this, params);

  this.setReactionData(params.reaction);

  var reaction = this.getReactionData();
  var point = reaction.getCenter();
  var latLng = this.getCustomMap().fromPointToLatLng(point);
  this.googleInfowindow = new google.maps.InfoWindow({
    content : this.content,
    position : latLng
  });
  this.googleInfowindow.open(this.getCustomMap().getGoogleMap(), this.getGoogleMarker());
}

ReactionInfoWindow.prototype = Object.create(AbstractInfoWindow.prototype);
ReactionInfoWindow.prototype.constructor = ReactionInfoWindow;

/**
 * Methods that creates and return html code with the content of the window.
 * 
 * @returns {String} representing html code for content of the info window
 */
ReactionInfoWindow.prototype.createContentDiv = function() {
  var self = this;
  var reaction = self.getReactionData();
  var result = document.createElement("div");
  var title = document.createElement("h3");
  title.innerHTML = reaction.getType() + ": " + reaction.getReactionId();
  result.appendChild(title);

  return Promise.resolve(result);
};

ReactionInfoWindow.prototype.init = function() {
  var self = this;
  return AbstractInfoWindow.prototype.init.call(this).then(function() {
    return self.update(self.getReactionData());
  });
};

/**
 * Returns array with data taken from all known {@link OverlayCollection}.
 * 
 * @returns array with data from {@link OverlayCollection}
 */
ReactionInfoWindow.prototype.getOverlaysData = function(general) {
  return this.getCustomMap().getTopMap().getOverlayDataForReaction(this.getReactionData(), general);
};

ReactionInfoWindow.prototype.getReactionData = function() {
  return this._reactionData;
};

ReactionInfoWindow.prototype.setReactionData = function(reactionData) {
  if (reactionData === undefined || reactionData === null) {
    throw new Error("Reaction must be specified");
  } else if (reactionData instanceof Reaction) {
    this._reactionData = reactionData;
  } else {
    throw new Error("Parameter must be of Reaction type, but found" + reactionData);
  }
};

module.exports = ReactionInfoWindow;
