"use strict";

require('../../mocha-config.js');

var Alias = require('../../../../main/js/map/data/Alias');
var GuiUtils = require('../../../../main/js/gui/leftPanel/GuiUtils');

var chai = require('chai');
var assert = chai.assert;
var logger = require('../../logger');

describe('GuiUtils', function() {

  it('contructor', function() {
    var map = helper.createCustomMap();
    helper.createSearchDbOverlay(map);

    new GuiUtils();
    assert.equal(logger.getWarnings().length, 0);
  });

  it('default', function() {
    var aliasObj = {
      symbol : "S1_SYMB",
      formerSymbols : [],
      modelId : 15781,
      synonyms : [ "syn44" ],
      description : "DESCRIPTION",
      type : "Simple molecule",
      abbreviation : "ABREVIATION",
      name : "s1",
      bounds : {
        x : 170.0,
        y : 171.5,
        width : 70.0,
        height : 25.0
      },
      formula : "FORMULA",
      id : 380217,
      references : [],
    };
    var alias = new Alias(aliasObj);

    var map = helper.createCustomMap();
    map.getModel().setId(15781);
    helper.createSearchDbOverlay(map);

    var guiUtils = new GuiUtils();
    guiUtils.setMap(map);

    var aliasDiv = guiUtils.createAliasElement({
      alias : alias
    });
    assert.ok(aliasDiv.innerHTML);
  });

  it('createLabelText for undefined', function() {
    var map = helper.createCustomMap();

    var guiUtils = new GuiUtils();
    guiUtils.setMap(map);

    var res = guiUtils.createLabelText();
    assert.notOk(res.innerHTML);
  });

  it('createSubMapLink', function() {
    var map = helper.createCustomMap();

    var guiUtils = new GuiUtils();
    guiUtils.setMap(map);

    var res = guiUtils.createSubMapLink("TEST", map.getId());
    assert.ok(res.innerHTML);
  });

  it('createPostTranslationalModifications', function() {
    var alias = helper.createAlias();
    alias.setOther({
      "modifications" : [ {
        "name" : "S63",
        "state" : "PHOSPHORYLATED"
      }, {
        "name" : "S73",
        "state" : "PHOSPHORYLATED"
      }, {
        "name" : "",
        "state" : "PHOSPHORYLATED"
      }, {
        "name" : null,
        "state" : "PHOSPHORYLATED"
      } ]
    });
    var utils = new GuiUtils(helper.getConfiguration());

    var res = utils.createPostTranslationalModifications("label", alias.getOther("modifications"));
    assert.ok(res.innerHTML);
    assert.ok(res.innerHTML.indexOf("phosphorylated") >= 0);
  });

  describe('createReactionElement', function() {
    it('type in desc', function() {
      var map = helper.createCustomMap();
      helper.createSearchDbOverlay(map);

      var guiUtils = new GuiUtils();
      guiUtils.setMap(map);

      var reaction = helper.createReaction(map);
      var reactionType = "XX TYPE";
      reaction.setType(reactionType);

      assert.ok(guiUtils.createReactionElement({
        reaction : reaction
      }).innerHTML.indexOf(reactionType) >= 0);
    });
  });

  describe('createAliasElement', function() {
    it('full name in desc', function() {
      var map = helper.createCustomMap();
      helper.createSearchDbOverlay(map);

      var guiUtils = new GuiUtils();
      guiUtils.setMap(map);
      var alias = helper.createAlias(map);

      alias.setFullName("xxx");
      assert.ok(guiUtils.createAliasElement({
        alias : alias
      }).innerHTML.indexOf("Full name") >= 0);

      alias.setFullName("");
      assert.ok(guiUtils.createAliasElement({
        alias : alias
      }).innerHTML.indexOf("Full name") === -1);
    });
  });

});
