"use strict";

require('../../mocha-config.js');

var MiRna = require('../../../../main/js/map/data/MiRna');
var MiRnaPanel = require('../../../../main/js/gui/leftPanel/MiRnaPanel');
var PanelControlElementType = require('../../../../main/js/gui/PanelControlElementType');

var chai = require('chai');
var assert = chai.assert;
var logger = require('../../logger');

describe('MiRnaPanel', function() {

  it('contructor', function() {
    var map = helper.createCustomMap();
    helper.createMiRnaDbOverlay(map);

    new MiRnaPanel({
      element : testDiv,
      customMap : map
    });
    assert.equal(logger.getWarnings().length, 0);
  });

  it('createMiRnaeader for empty', function() {
    var map = helper.createCustomMap();
    helper.createMiRnaDbOverlay(map);

    var panel = new MiRnaPanel({
      element : testDiv,
      customMap : map
    });

    assert.ok(panel.createPreamble().innerHTML.indexOf("NOT FOUND") > 0);
  });

  it('createMiRnaeader for empty', function() {
    var map = helper.createCustomMap();
    helper.createMiRnaDbOverlay(map);

    var panel = new MiRnaPanel({
      element : testDiv,
      customMap : map
    });

    assert.ok(panel.createPreamble(new MiRna()).innerHTML.indexOf("NOT FOUND") > 0);
  });

  it('on searchResults changed', function() {
    var map = helper.createCustomMap();
    map.getModel().setId(15781);
    var miRnaDbOverlay = helper.createMiRnaDbOverlay(map);

    new MiRnaPanel({
      element : testDiv,
      customMap : map
    });

    return miRnaDbOverlay.searchByQuery("hsa-miR-125a-3p").then(function() {
      assert.equal(logger.getWarnings().length, 0);
      assert.ok(testDiv.innerHTML.indexOf("hsa-miR-125a-3p") >= 0);
    });
  });

  it('searchByQuery', function() {
    var map = helper.createCustomMap();
    map.getModel().setId(15781);
    helper.createMiRnaDbOverlay(map);

    var panel = new MiRnaPanel({
      element : testDiv,
      customMap : map,
    });

    panel.getControlElement(PanelControlElementType.SEARCH_INPUT).value = "hsa-miR-125a-3p";

    return panel.searchByQuery().then(function() {
      assert.equal(logger.getWarnings().length, 0);
      assert.ok(testDiv.innerHTML.indexOf("http://www.mirbase.org/cgi-bin/mirna_entry.pl?acc") >= 0);
    });
  });

});
