"use strict";

var ReactionSurface = require('../../../../main/js/map/surface/ReactionSurface');

var assert = require('assert');

var logger = require('../../logger');

describe('ReactionSurface', function() {
  it("Constructor", function() {
    var map = helper.createAbstractCustomMap();
    var reaction = helper.createReaction(map);
    var layoutReaction = helper.createLayoutReaction(reaction);

    var reactionOverlay = new ReactionSurface({
      overlayData : layoutReaction,
      reaction : reaction,
      map : map,
      customized : false
    });

    assert.ok(reactionOverlay.getGoogleMapObjects().length > 0);
    assert.ok(reactionOverlay.getColor());
    assert.ok(reactionOverlay.getWidth());
    assert.ok(reactionOverlay.getBounds());
    assert.ok(reactionOverlay.getReactionData());
    assert.ok(reactionOverlay.getCustomMap());
    assert.ok(reactionOverlay.getId());

  });

  it("show", function() {
    var map = helper.createCustomMap();
    var reaction = helper.createReaction(map);
    var layoutReaction = helper.createLayoutReaction(reaction);

    var reactionOverlay = new ReactionSurface({
      overlayData : layoutReaction,
      reaction : reaction,
      map : map,
      customized : false
    });

    reactionOverlay.hide();
    assert.equal(reactionOverlay.isShown(), false);

    reactionOverlay.show();
    assert.ok(reactionOverlay.isShown());

    assert.equal(logger.getWarnings().length, 0);

    reactionOverlay.show();
    assert.ok(reactionOverlay.isShown());
    assert.equal(logger.getWarnings().length, 1);

  });

  it("changedToCustimzed", function() {
    var map = helper.createCustomMap();
    var reaction = helper.createReaction(map);
    var layoutReaction = helper.createLayoutReaction(reaction);

    var reactionOverlay = new ReactionSurface({
      overlayData : layoutReaction,
      reaction : reaction,
      map : map,
      customized : false
    });

    reactionOverlay.changedToCustimzed();
    assert.ok(reactionOverlay.isCustomized());

  });

  it("createline", function() {
    var map = helper.createAbstractCustomMap();

    var line = ReactionSurface.createLine({
      start : {
        x : 0,
        y : 10
      },
      end : {
        x : 100,
        y : 100
      },
    }, "#000000", 3, map);

    assert.ok(line);

  });
});
