"use strict";

var Promise = require("bluebird");
var Cookies = require('js-cookie');

var Helper = require('./Helper');

// GLOBAL configuration
global.navigator = {
  userAgent : 'node.js',
  appName : 'MinervaUnitTest',
  appVersion : '0.0.1',

};

var jsdom = require('jsdom');
global.dom = new jsdom.JSDOM();
global.window = global.dom.window;
global.document = window.document;

global.$ = require('jQuery');
global.jQuery = $;
global.window.$ = $;

// additions to jsdom implementations:
global.Option = window.Option;

var originalCreateElement = document.createElement;
document.createElement = function(arg) {
  var result = originalCreateElement.call(this, arg);
  // woraround for: https://github.com/tmpvar/jsdom/issues/961
  if ("li" === arg) {
    result.dataset = [];
  }
  return result;
};

window.open = function() {
  var result = {};
  result.focus = function() {

  };
  return result;
};

// pileup is using heavily some browser defined javascript
var pileup = require('pileup');
pileup.create = function() {
  return {
    destroy : function() {
    }
  };
};
pileup.formats.twoBit = function() {
  return {};
};
pileup.formats.bigBed = function() {
  return {};
};
// ---

require('jquery-ui-dist/jquery-ui.js');

require("bootstrap");

require('datatables.net')(window, $);

global.google = require('./google-map-mock');

global.GuiConnector = require('./GuiConnector-mock');
global.ServerConnector = require('./ServerConnector-mock');

// -----------------------------

var logger = require('./logger');

function removeCookies() {
  var cookies = Cookies.get();
  for ( var cookie in cookies) {
    Cookies.remove(cookie);
  }
}

function mockBootstrap() {
  $.fn.typeahead = function() {
    logger.debug("Mock typeahead function call");
  };
}

before(function() {
  Promise.longStackTraces();

  mockBootstrap();
});

beforeEach(function() {
  window.onresize = undefined;

  logger.flushBuffer();

  removeCookies();

  ServerConnector.init();

  ServerConnector.getSessionData(null).setToken("MOCK_TOKEN_ID");
  ServerConnector.getSessionData(null).setLogin("anonymous");


  global.testDiv = document.createElement("div");
  global.testDiv.id = "test";
  document.body.appendChild(testDiv);

  return ServerConnector.getConfiguration().then(function(configuration) {
    global.helper = new Helper(configuration);
    helper.setUrl("http://test/");
    GuiConnector.init();
  });

});

afterEach(function() {
  document.body.removeChild(global.testDiv);
  delete global.testDiv;
});
